import json
from time import time

from DIRAC import gLogger, gConfig
from DIRAC.WorkloadManagementSystem.Client.WMSAdministratorClient import WMSAdministratorClient

from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler

# Dictionary of country code top-level domain (ccTLD) and corresponding country name
COUNTRIES = {
    "af": "Afghanistan",
    "al": "Albania",
    "dz": "Algeria",
    "as": "American Samoa",
    "ad": "Andorra",
    "ao": "Angola",
    "ai": "Anguilla",
    "aq": "Antarctica",
    "ag": "Antigua and Barbuda",
    "ar": "Argentina",
    "am": "Armenia",
    "aw": "Aruba",
    "au": "Australia",
    "at": "Austria",
    "az": "Azerbaijan",
    "bs": "Bahamas",
    "bh": "Bahrain",
    "bd": "Bangladesh",
    "bb": "Barbados",
    "by": "Belarus",
    "be": "Belgium",
    "bz": "Belize",
    "bj": "Benin",
    "bm": "Bermuda",
    "bt": "Bhutan",
    "bo": "Bolivia",
    "ba": "Bosnia and Herzegowina",
    "bw": "Botswana",
    "bv": "Bouvet Island",
    "br": "Brazil",
    "io": "British Indian Ocean Territory",
    "bn": "Brunei Darussalam",
    "bg": "Bulgaria",
    "bf": "Burkina Faso",
    "bi": "Burundi",
    "kh": "Cambodia",
    "cm": "Cameroon",
    "ca": "Canada",
    "cv": "Cape Verde",
    "ky": "Cayman Islands",
    "cf": "Central African Republic",
    "td": "Chad",
    "cl": "Chile",
    "cn": "China",
    "cx": "Christmas Island",
    "cc": "Cocos Islands",
    "co": "Colombia",
    "km": "Comoros",
    "cg": "Congo",
    "cd": "Congo",
    "ck": "Cook Islands",
    "cr": "Costa Rica",
    "ci": "Cote D'Ivoire",
    "hr": "Croatia",
    "cu": "Cuba",
    "cy": "Cyprus",
    "cz": "Czech Republic",
    "dk": "Denmark",
    "dj": "Djibouti",
    "dm": "Dominica",
    "do": "Dominican Republic",
    "tp": "East Timor",
    "ec": "Ecuador",
    "eg": "Egypt",
    "sv": "El Salvador",
    "gq": "Equatorial Guinea",
    "er": "Eritrea",
    "ee": "Estonia",
    "et": "Ethiopia",
    "fk": "Falkland Islands",
    "fo": "Faroe Islands",
    "fj": "Fiji",
    "fi": "Finland",
    "fr": "France",
    "fx": "France, metropolitan",
    "gf": "French Guiana",
    "pf": "French Polynesia",
    "tf": "French Southern Territories",
    "ga": "Gabon",
    "gm": "Gambia",
    "ge": "Georgia",
    "de": "Germany",
    "gh": "Ghana",
    "gi": "Gibraltar",
    "gr": "Greece",
    "gl": "Greenland",
    "gd": "Grenada",
    "gp": "Guadeloupe",
    "gu": "Guam",
    "gt": "Guatemala",
    "gn": "Guinea",
    "gw": "Guinea-Bissau",
    "gy": "Guyana",
    "ht": "Haiti",
    "hm": "Heard and Mc Donald Islands",
    "va": "Vatican City",
    "hn": "Honduras",
    "hk": "Hong Kong",
    "hu": "Hungary",
    "is": "Iceland",
    "in": "India",
    "id": "Indonesia",
    "ir": "Iran",
    "iq": "Iraq",
    "ie": "Ireland",
    "il": "Israel",
    "it": "Italy",
    "jm": "Jamaica",
    "jp": "Japan",
    "jo": "Jordan",
    "kz": "Kazakhstan",
    "ke": "Kenya",
    "ki": "Kiribati",
    "kp": "Korea",
    "kr": "Korea",
    "kw": "Kuwait",
    "kg": "Kyrgyzstan",
    "la": "Lao",
    "lv": "Latvia",
    "lb": "Lebanon",
    "ls": "Lesotho",
    "lr": "Liberia",
    "ly": "Libyan",
    "li": "Liechtenstein",
    "lt": "Lithuania",
    "lu": "Luxembourg",
    "mo": "Macau",
    "mk": "Macedonia",
    "mg": "Madagascar",
    "mw": "Malawi",
    "my": "Malaysia",
    "mv": "Maldives",
    "ml": "Mali",
    "mt": "Malta",
    "mh": "Marshall Islands",
    "mq": "Martinique",
    "mr": "Mauritania",
    "mu": "Mauritius",
    "yt": "Mayotte",
    "mx": "Mexico",
    "fm": "Micronesia",
    "md": "Moldova",
    "mc": "Monaco",
    "mn": "Mongolia",
    "ms": "Montserrat",
    "ma": "Morocco",
    "mz": "Mozambique",
    "mm": "Myanmar",
    "na": "Namibia",
    "nr": "Nauru",
    "np": "Nepal",
    "nl": "Netherlands",
    "an": "Netherlands Antilles",
    "nc": "New Caledonia",
    "nz": "New Zealand",
    "ni": "Nicaragua",
    "ne": "Niger",
    "ng": "Nigeria",
    "nu": "Niue",
    "nf": "Norfolk Island",
    "mp": "Northern Mariana Islands",
    "no": "Norway",
    "om": "Oman",
    "pk": "Pakistan",
    "pw": "Palau",
    "pa": "Panama",
    "pg": "Papua New Guinea",
    "py": "Paraguay",
    "pe": "Peru",
    "ph": "Philippines",
    "pn": "Pitcairn",
    "pl": "Poland",
    "pt": "Portugal",
    "pr": "Puerto Rico",
    "qa": "Qatar",
    "re": "Reunion",
    "ro": "Romania",
    "ru": "Russia",
    "rw": "Rwanda",
    "kn": "Saint Kitts and Nevis",
    "lc": "Saint Lucia",
    "vc": "Saint Vincent and the Grenadines",
    "ws": "Samoa",
    "sm": "San Marino",
    "st": "Sao Tome and Principe",
    "sa": "Saudi Arabia",
    "sn": "Senegal",
    "sc": "Seychelles",
    "sl": "Sierra Leone",
    "sg": "Singapore",
    "sk": "Slovakia",
    "si": "Slovenia",
    "sb": "Solomon Islands",
    "so": "Somalia",
    "za": "South Africa",
    "gs": "South Georgia and the South Sandwich Islands",
    "es": "Spain",
    "lk": "Sri Lanka",
    "sh": "St. Helena",
    "pm": "St. Pierre and Miquelon",
    "sd": "Sudan",
    "sr": "Suriname",
    "sj": "Svalbard and Jan Mayen Islands",
    "sz": "Swaziland",
    "se": "Sweden",
    "ch": "Switzerland",
    "sy": "Syrian Arab Republic",
    "tw": "Taiwan",
    "tj": "Tajikistan",
    "tz": "Tanzania",
    "th": "Thailand",
    "tg": "Togo",
    "tk": "Tokelau",
    "to": "Tonga",
    "tt": "Trinidad and Tobago",
    "tn": "Tunisia",
    "tr": "Turkey",
    "tm": "Turkmenistan",
    "tc": "Turks and Caicos Islands",
    "tv": "Tuvalu",
    "ug": "Uganda",
    "ua": "Ukraine",
    "ae": "United Arab Emirates",
    "gb": "United Kingdom",
    "uk": "United Kingdom",
    "us": "United States",
    "um": "United States Minor Outlying Islands",
    "uy": "Uruguay",
    "uz": "Uzbekistan",
    "vu": "Vanuatu",
    "ve": "Venezuela",
    "vn": "Viet Nam",
    "vg": "Virgin Islands (British)",
    "vi": "Virgin Islands (U.S.)",
    "wf": "Wallis and Futuna Islands",
    "eh": "Western Sahara",
    "ye": "Yemen",
    "yu": "Yugoslavia",
    "zm": "Zambia",
    "zw": "Zimbabwe",
    "su": "Soviet Union",
}


class JobSummaryHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "all"

    pageNumber = 0
    numberOfJobs = 25
    globalSort = [["GridSite", "ASC"]]

    def web_getSelectionData(self):
        callback = {}

        result = WMSAdministratorClient().getSiteSummarySelectors()
        gLogger.info("\033[0;31m ++++++: \033[0m %s" % result)
        if result["OK"]:
            result = result["Value"]
            if len(result.get("Status", [])) > 0:
                status = [[str("All")]]
                for i in result["Status"]:
                    status.append([str(i)])
            else:
                status = [["Nothing to display"]]
            callback["status"] = status
            if len(result.get("GridType", [])) > 0:
                gridtype = [[str("All")]]
                for i in result["GridType"]:
                    gridtype.append([str(i)])
            else:
                gridtype = [["Nothing to display"]]
            callback["gridtype"] = gridtype
            if len(result.get("MaskStatus", [])) > 0:
                maskstatus = [[str("All")]]
                for i in result["MaskStatus"]:
                    maskstatus.append([str(i)])
            else:
                maskstatus = [["Nothing to display"]]
            callback["maskstatus"] = maskstatus
            if len(result.get("Site", [])) > 0:
                s = list(result["Site"])
                tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
                site = [["All"]]
                for i in tier1:
                    site.append([str(i)])
                for i in s:
                    if i not in tier1:
                        site.append([str(i)])
            else:
                site = [["Error during RPC call"]]
            callback["site"] = site
            if len(result.get("Country", [])) > 0:
                country = [["All"]]
                countryCode = COUNTRIES
                for i in result["Country"]:
                    if i in countryCode:
                        j = countryCode[i]
                        country.append([str(j)])
            else:
                country = [["Nothing to display"]]
            country.sort()
            callback["country"] = country
        else:
            callback["status"] = [["Error during RPC call"]]
            callback["gridtype"] = [["Error during RPC call"]]
            callback["maskstatus"] = [["Error during RPC call"]]
            callback["site"] = [["Error during RPC call"]]
            callback["country"] = [["Error during RPC call"]]
        return callback

    def web_getData(self):
        pagestart = time()
        result = self.__request()
        gLogger.always("getSiteSummaryWeb(%s,%s,%s,%s)" % (result, self.globalSort, self.pageNumber, self.numberOfJobs))
        retVal = WMSAdministratorClient().getSiteSummaryWeb(result, [], self.pageNumber, self.numberOfJobs)
        gLogger.always("\033[0;31m YO: \033[0m", result)
        if retVal["OK"]:
            if retVal["Value"].get("TotalRecords", 0) > 0:
                if "ParameterNames" in retVal["Value"] and "Records" in retVal["Value"]:
                    if len(retVal["Value"].get("ParameterNames", [])) > 0:
                        if len(retVal["Value"].get("Records", [])) > 0:
                            result = []
                            jobs = retVal["Value"]["Records"]
                            head = retVal["Value"]["ParameterNames"]
                            headLength = len(head)
                            countryCode = COUNTRIES
                            for i in jobs:
                                tmp = {head[j]: i[j] for j in range(headLength)}
                                tmp["FullCountry"] = countryCode[i[2]] if i[2] in countryCode else "Unknown"
                                result.append(tmp)
                            total = retVal["Value"]["TotalRecords"]
                            if "Extras" in retVal["Value"]:
                                extra = retVal["Value"]["Extras"]
                                result = {"success": "true", "result": result, "total": total, "extra": extra}
                            else:
                                result = {"success": "true", "result": result, "total": total}
                        else:
                            result = {"success": "false", "result": "", "error": "There are no data to display"}
                    else:
                        result = {"success": "false", "result": "", "error": "ParameterNames field is undefined"}
                else:
                    result = {"success": "false", "result": "", "error": "Data structure is corrupted"}
            else:
                result = {"success": "false", "result": "", "error": "There were no data matching your selection"}
        else:
            gLogger.always("- E R R O R -")
            result = {"success": "false", "error": retVal["Message"]}
        gLogger.info("\033[0;31m SITESUMMARY INDEX REQUEST: \033[0m %s" % (time() - pagestart))
        return result

    def __request(self):
        req = {}

        if "id" in self.request.arguments:
            jobids = list(json.loads(self.get_argument("id")))
            if jobids:
                req["JobID"] = jobids

        elif len(self.get_argument("expand", [])) > 0:
            self.globalSort = [["GridSite", "ASC"]]
            self.numberOfJobs = 500
            self.pageNumber = 0
            req["ExpandSite"] = self.get_argument("expand", "")
        else:
            self.pageNumber = 0
            self.numberOfJobs = 500
            countries = list(json.loads(self.get_argument("country", "[]")))
            if countries:
                # Return the dictionary of country names and
                # corresponding country code top-level domain (ccTLD)
                code = dict(zip(COUNTRIES.values(), COUNTRIES))
                newValue = [code[i] for i in countries if i in code]
                req["Country"] = newValue

            site = list(json.loads(self.get_argument("site", "[]")))
            if site:
                req["Site"] = site

            status = list(json.loads(self.get_argument("status", "[]")))
            if status:
                req["Status"] = status

            maskstatus = list(json.loads(self.get_argument("maskstatus", "[]")))
            if maskstatus:
                req["MaskStatus"] = maskstatus

            gridtype = list(json.loads(self.get_argument("gridtype", "[]")))
            owner = list(json.loads(self.get_argument("owner", "[]")))
            if gridtype:
                req["GridType"] = gridtype
            elif owner:
                req["Owner"] = owner

            if self.get_argument("date", ""):
                req["LastUpdate"] = self.get_argument("date")

        gLogger.info("REQUEST:", req)
        return req



""" WebApp handler for Downtimes WebApp
"""

import json
from datetime import datetime

from DIRAC import gLogger
from DIRAC.ResourceStatusSystem.Client.PublisherClient import PublisherClient
from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler, WErr


class DowntimesHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "authenticated"

    # Publisher client
    pubClient = None

    @classmethod
    def initializeHandler(cls, serviceInfo):
        """This may be overwritten when you write a DIRAC service handler
        And it must be a class method. This method is called only one time,
        at the first request

        :param dict serviceInfo: infos about services
        """
        cls.pubClient = PublisherClient()

    def web_getSelectionData(self):
        """Get selection data

        :return: dict
        """
        callback = {"name": set(), "severity": set(), "sites": set()}

        downtimes = self.pubClient.getCachedDowntimes(None, None, None, None)

        if downtimes["OK"]:
            dtList = [dict(zip(downtimes["Columns"], dt)) for dt in downtimes["Value"]]

            for dt in dtList:
                callback["name"].add(dt["Name"])
                callback["severity"].add(dt["Severity"])

        sites = self.pubClient.getSites()
        if sites["OK"]:
            callback["site"] = sites["Value"]

        for key, value in callback.items():
            callback[key] = [[item] for item in list(value)]
            callback[key] = [["All"]] + callback[key]

        callback["view"] = [["tabular"], ["availability"]]
        return callback

    def web_getDowntimesData(
        self, name=None, severity=None, site=None, startDate=None, startTime=None, endDate=None, endTime=None
    ):
        """Get downtimes data

        :param str name: name
        :param str severity: severity
        :param str site: site
        :param str startDate: start date
        :param str startTime: start time
        :param str endDate: end date
        :param str endTime: end time

        :return: dict
        """
        requestParams = {
            "name": list(json.loads(name)) if name else [],
            "site": list(json.loads(site)) if site else [],
            "severity": list(json.loads(severity)) if severity else [],
        }

        requestParams["startDate"] = datetime.utcnow()
        if startDate and startTime:
            requestParams["startDate"] = datetime.strptime("%s %s" % (startDate, startTime), "%Y-%m-%d %H:%M")

        requestParams["endDate"] = datetime.utcnow()
        if endDate and endTime:
            requestParams["endDate"] = datetime.strptime("%s %s" % (endDate, endTime), "%Y-%m-%d %H:%M")

        gLogger.info("Request parameters:", requestParams)

        retVal = self.pubClient.getSitesResources(requestParams["site"])

        if not retVal["OK"]:
            raise WErr.fromSERROR(retVal)
        sitesResources = retVal["Value"]

        names = []
        if requestParams["site"]:
            for _site, resources in list(sitesResources.items()):
                names += resources["ces"]
                names += resources["ses"]

        downtimes = self.pubClient.getCachedDowntimes(None, None, names, requestParams["severity"])
        if not downtimes["OK"]:
            raise WErr.fromSERROR(downtimes)

        dtList = [dict(zip(downtimes["Columns"], dt)) for dt in downtimes["Value"]]

        for dt in dtList:
            dt["Site"] = "Unknown"

            for site, resources in list(sitesResources.items()):
                if dt["Name"] in resources["ces"] + resources["ses"]:
                    dt["Site"] = site
                    break

            dt["StartDate"] = str(dt["StartDate"])
            dt["EndDate"] = str(dt["EndDate"])

        return {"success": "true", "result": dtList, "total": len(dtList)}



import os
import time
import random
import shutil
import zipfile

from DIRAC import gLogger, S_ERROR, S_OK
from DIRAC.Core.Utilities import Time
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getVOForGroup

from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler, FileResponse


class FileCatalogHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "authenticated"

    # Supported operands
    __operands = ["in", "nin", "=", "!=", ">=", "<=", ">", "<"]

    def initializeRequest(self):
        """Called at every request, may be overwritten in your handler."""
        self.user = self.getUserName()
        self.group = self.getUserGroup()
        self.vo = getVOForGroup(self.group)
        self.fc = FileCatalog(vo=self.vo)

    def web_getSelectedFiles(self, archivePath=None, lfnPath=None):
        """Method to get the selected file(s)

        :param str archivePath: archive path
        :param str lfnPath: path LFN

        :return: dict
        """
        if lfnPath is None:
            lfnPath = ""

        # First pass: download files and check for the success
        if not archivePath:
            tmpdir = "/tmp/" + str(time.time()) + str(random.random())
            dataMgr = DataManager(vo=self.vo)
            if not os.path.isdir(tmpdir):
                os.makedirs(tmpdir)
            os.chdir(tmpdir)
            for lfn in lfnPath.split(","):
                gLogger.always("Data manager get file %s" % lfn)
                last_slash = lfn.rfind("/")
                pos_relative = lfn.find("/")
                pos_relative = lfn.find("/", pos_relative + 1)
                pos_relative = lfn.find("/", pos_relative + 1)
                pos_relative = pos_relative
                pathInZip = lfn[pos_relative:last_slash]
                tmpPathInZip = tmpdir + pathInZip
                gLogger.always("path in zip %s" % tmpPathInZip)
                if not os.path.isdir(tmpPathInZip):
                    os.makedirs(tmpPathInZip)
                result = dataMgr.getFile(str(lfn), destinationDir=str(tmpPathInZip))
                if not result["OK"]:
                    gLogger.error("Error getting while getting files", result["Message"])
                    shutil.rmtree(tmpdir)
                    return {"success": "false", "error": result["Message"], "lfn": lfn}

            # make zip file
            zipname = tmpdir.split("/")[-1] + ".zip"
            archivePath = "/tmp/" + zipname
            zFile = zipfile.ZipFile(archivePath, "w")
            gLogger.always("zip file", archivePath)
            gLogger.always("start walk in tmpdir", tmpdir)
            for absolutePath, dirs, files in os.walk(tmpdir):
                gLogger.always("absolute path", absolutePath)
                gLogger.always("files", files)
                for filename in files:
                    # relative path form tmpdir current chdir
                    pos_relative = absolutePath.find("/")
                    pos_relative = absolutePath.find("/", pos_relative + 1)
                    pos_relative = absolutePath.find("/", pos_relative + 1)
                    pos_relative = absolutePath.find("/", pos_relative + 1)
                    pos_relative = pos_relative + 1
                    relativePath = absolutePath[pos_relative:]
                    gLogger.always("relativePath %s, file %s" % (relativePath, filename))
                    zFile.write(os.path.join(absolutePath, filename))
            zFile.close()
            shutil.rmtree(tmpdir)
            return {"success": "true", "archivePath": archivePath}

        # Second pass: deliver the requested archive
        # read zip file
        with open(archivePath, "rb") as archive:
            data = archive.read()
        # cleanup
        os.remove(archivePath)
        return FileResponse(data, os.path.basename(archivePath))

    def web_getMetadataFields(self):
        """Method to read all the available fields possible for defining a query

        :return: dict
        """
        callback = {}
        result = self.__getMetadataFields()
        if not result["OK"]:
            gLogger.error(result["Message"])
            return {"success": "false", "error": result["Message"]}
        filemeta, dirmeta = result["Value"]
        for key in filemeta:
            callback[key] = "label"
        gLogger.debug("getSelectorGrid: FileMetaFields callback %s" % callback)
        for key, value in dirmeta.items():
            callback[key] = value.lower()
        gLogger.debug("getSelectorGrid: Resulting callback %s" % callback)
        return {"success": "true", "result": callback}

    def web_getQueryData(self, lfnPath=None, **kwargs):
        """Method to read all the available options for a metadata field

        :param str lfnPath: path LFN

        :return: dict
        """
        if lfnPath is None:
            lfnPath = "/"

        try:
            compat = {}
            for key in kwargs:
                parts = key.split(".")
                if len(parts) != 3:
                    continue
                _, name, sign = parts

                if not name:
                    continue

                value = kwargs[key].split("|")

                # check existence of the 'name' section
                if name not in compat:
                    compat[name] = {}

                # check existence of the 'sign' section
                if sign not in compat[name]:
                    if value[0] == "v":
                        compat[name][sign] = ""
                    elif value[0] == "s":
                        compat[name][sign] = []

                if value[0] == "v":
                    compat[name][sign] = value[1]
                elif value[0] == "s":
                    compat[name][sign] += value[1].split(":::")

        except Exception as e:
            return {"success": "false", "error": f"Metadata query error: {e!r}"}

        gLogger.always(compat)

        result = self.fc.getCompatibleMetadata(compat, lfnPath)
        gLogger.always(result)

        if not result["OK"]:
            return {"success": "false", "error": result["Message"]}
        return {"success": "true", "result": result["Value"]}

    def web_getFilesData(self, limit=25, start=0, lfnPath=None, **kwargs):
        """Get files data

        :param int limit: limit
        :param int start: start
        :param str lfnPath: path

        :return: dict
        """
        req = {"selection": {}, "path": "/"}

        result = self.__getMetadataFields()
        if not result["OK"]:
            gLogger.error("request:", result["Message"])
        else:
            filemeta, dirmeta = result["Value"]
            meta = [k for k in dirmeta]
            gLogger.always("request: metafields:", meta)

            for param in kwargs:
                tmp = param.split(".")
                if len(tmp) != 3:
                    continue
                _, name, logic = tmp
                value = kwargs[param].split("|")
                if logic not in self.__operands:
                    gLogger.always(f"Operand '{logic}' is not supported")
                    continue

                if name in meta:
                    # check existence of the 'name' section
                    if name not in req["selection"]:
                        req["selection"][name] = {}

                    # check existence of the 'sign' section
                    if logic not in req["selection"][name]:
                        if value[0] == "v":
                            req["selection"][name][logic] = ""
                        elif value[0] == "s":
                            req["selection"][name][logic] = []

                    if value[0] == "v":
                        req["selection"][name][logic] = value[1]
                    elif value[0] == "s":
                        req["selection"][name][logic] += value[1].split(":::")
            if lfnPath:
                req["path"] = lfnPath

        gLogger.debug("submit: incoming request %s" % req)
        result = self.fc.findFilesByMetadataWeb(req["selection"], req["path"], start, limit)
        gLogger.debug("submit: result of findFilesByMetadataDetailed", result)
        if not result["OK"]:
            gLogger.error("submit:", result["Message"])
            return {"success": "false", "error": result["Message"]}
        result = result["Value"]

        if not len(result) > 0:
            return {"success": "true", "result": [], "total": 0, "date": "-"}

        total = result["TotalRecords"]
        result = result["Records"]

        callback = list()
        for key, value in result.items():
            size = ""
            if "Size" in value:
                size = value["Size"]

            date = ""
            if "CreationDate" in value:
                date = str(value["CreationDate"])

            meta = ""
            if "Metadata" in value:
                m = value["Metadata"]
                meta = "; ".join(["%s: %s" % (i, j) for (i, j) in m.items()])

            dirnameList = key.split("/")
            dirname = "/".join(dirnameList[: len(dirnameList) - 1])
            filename = dirnameList[len(dirnameList) - 1 :]

            callback.append(
                {
                    "fullfilename": key,
                    "dirname": dirname,
                    "filename": filename,
                    "date": date,
                    "size": size,
                    "metadata": meta,
                }
            )
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        return {"success": "true", "result": callback, "total": total, "date": timestamp}

    def web_getMetadataFilesInFile(self, selection="", lfnPath=None):
        """Get metadata files

        :param str selection: selection
        :param str lfnPath: path

        :return: dict
        """
        req = {"selection": {}, "path": "/"}
        result = self.__getMetadataFields()
        if not result["OK"]:
            gLogger.error("request:", result["Message"])
        else:
            filemeta, dirmeta = result["Value"]
            meta = [k for k in dirmeta]
            gLogger.always("request: metafields: ", meta)

            selectionElems = selection.split("<|>")

            gLogger.debug("request: THISSSS", selection)

            for param in selectionElems:
                tmp = str(param).split("|")

                if len(tmp) != 4:
                    continue

                name = tmp[0]
                logic = tmp[1]

                if logic not in self.__operands:
                    gLogger.always(f"Operand '{logic}' is not supported")
                    continue

                if name in meta:
                    # check existence of the 'name' section
                    if name not in req["selection"]:
                        req["selection"][name] = dict()

                    # check existence of the 'sign' section
                    if logic not in req["selection"][name]:
                        if tmp[2] == "v":
                            req["selection"][name][logic] = ""
                        elif tmp[2] == "s":
                            req["selection"][name][logic] = []

                    if tmp[2] == "v":
                        req["selection"][name][logic] = tmp[3]
                    elif tmp[2] == "s":
                        req["selection"][name][logic] += tmp[3].split(":::")
            if lfnPath:
                req["path"] = lfnPath

        gLogger.debug("submit: incoming request", req)
        result = self.fc.findFilesByMetadata(req["selection"], req["path"])
        if not result["OK"]:
            gLogger.error("submit: %s" % result["Message"])
            return {"success": "false", "error": result["Message"]}

        return FileResponse("\n".join([fileName for fileName in result["Value"]]), str(req))

    def web_getSubnodeFiles(self, lfnPath):
        """Get subnode files

        :param str lfnPath: path

        :return: dict
        """
        result = self.fc.listDirectory(lfnPath, False)
        if not result["OK"]:
            gLogger.error("submit:", result["Message"])
            return {"success": "false", "error": result["Message"]}

        filesData = result["Value"]["Successful"][lfnPath]["Files"]
        dirData = result["Value"]["Successful"][lfnPath]["SubDirs"]

        retData = []

        for entryName in dirData:
            nodeDef = {"text": entryName.split("/")[-1]}
            nodeDef["leaf"] = False
            nodeDef["expanded"] = False
            retData.append(nodeDef)

        for entryName in filesData:
            nodeDef = {"text": entryName.split("/")[-1]}
            nodeDef["leaf"] = True
            retData.append(nodeDef)

        retData = sorted(retData, key=lambda node: node["text"].upper())
        return {"success": "true", "nodes": retData}

    def __getMetadataFields(self):
        """Helper method to get metadata fields

        :return: S_OK(tuple)/S_ERROR()
        """
        result = self.fc.getMetadataFields()
        if not result["OK"]:
            return result
        if "FileMetaFields" not in result["Value"]:
            return S_ERROR("Service response has no FileMetaFields key. Return empty dict")
        if "DirectoryMetaFields" not in result["Value"]:
            return S_ERROR("Service response has no DirectoryMetaFields key. Return empty dict")
        return S_OK((result["Value"]["FileMetaFields"], result["Value"]["DirectoryMetaFields"]))



import collections
import json

from DIRAC import gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.ResourceStatusSystem.Client.PublisherClient import PublisherClient
from DIRAC.ResourceStatusSystem.PolicySystem.StateMachine import RSSMachine

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class SummaryHandlerMix(WebHandler):
    AUTH_PROPS = "all"
    ELEMENT_TYPE = None

    def _getSelectionData(self) -> dict:
        """It returns the possible selection data"""
        callback = {"name": set(), "elementType": set(), "status": set(), "statusType": set(), "tokenOwner": set()}

        pub = PublisherClient()

        gLogger.info("Arguments to web_getSelectionData", repr(self.request.arguments))
        elementStatuses = pub.getElementStatuses(self.ELEMENT_TYPE, None, None, None, None, None)

        if elementStatuses["OK"]:

            for elementStatus in elementStatuses["Value"]:
                callback["status"].add(elementStatus[2])
                callback["name"].add(elementStatus[0])
                callback["elementType"].add(elementStatus[6])
                callback["statusType"].add(elementStatus[1])
                callback["tokenOwner"].add(elementStatus[8])

        for key, value in callback.items():

            callback[key] = sorted([[item] for item in list(value)])
            callback[key] = [["All"]] + callback[key]

        return callback

    def combine(self, elementValues: list) -> dict:
        """Helper method to combine values

        :param elementValues: values
        """

        statuses = [element["Status"] for element in elementValues]

        statusSet = set(statuses)

        if len(statusSet) == 1:
            status = statusSet.pop()
            reason = "All %s" % status

        else:

            if set(["Active", "Degraded"]) & set(statusSet):
                status = "Degraded"
                reason = "Not completely active"

            else:
                status = "Banned"
                reason = "Not usable"

        #      if set( [ 'Unknown','Active', 'Degraded' ] ) & set( statusSet ):
        #        for upStatus in [ 'Active', 'Degraded' ]:
        #          if upStatus in statusSet:
        #            status = upStatus
        #            reason = '%d %s' % ( statuses.count( upStatus ), upStatus )
        #            break
        #      else:
        #        for downStatus in [ 'Unknown','Probing','Banned','Error' ]:
        #          if downStatus in statusSet:
        #            status = downStatus
        #            reason = '%d %s' % ( statuses.count( downStatus ), downStatus )
        #            break

        # Make a copy
        combined = {}
        combined.update(elementValues[0])
        combined["StatusType"] = "%d elements" % len(statuses)
        combined["Status"] = status
        combined["Reason"] = reason
        combined["DateEffective"] = ""
        combined["LastCheckTime"] = ""
        combined["TokenOwner"] = ""
        combined["TokenExpiration"] = ""

        return combined

    def _expand(self) -> dict:
        """This method handles the POST requests"""

        requestParams = self._requestParams()
        gLogger.info(requestParams)

        pub = PublisherClient()

        elements = pub.getElementStatuses(self.ELEMENT_TYPE, requestParams["name"], None, None, None, None)
        if not elements["OK"]:
            return {"success": "false", "error": elements["Message"]}

        elementList = [dict(zip(elements["Columns"], element)) for element in elements["Value"]]
        for element in elementList:
            element["DateEffective"] = str(element["DateEffective"])
            element["LastCheckTime"] = str(element["LastCheckTime"])
            element["TokenExpiration"] = str(element["TokenExpiration"])

        return {"success": "true", "result": elementList, "total": len(elementList)}

    def _action(self) -> dict:
        """Do action requested from the web portal."""
        requestParams = self._requestParams()
        if not requestParams.get("action"):
            return {"success": "false", "error": "Missing action"}

        # pylint does not understand the action entry is not None any more
        actionName = requestParams["action"][0]  # pylint: disable=unsubscriptable-object

        methodName = actionName
        if not actionName.startswith("set"):
            methodName = f"_get{actionName}"

        try:
            return getattr(self, methodName)(requestParams)
        except AttributeError:
            return {"success": "false", "error": f"bad action {actionName}"}

    def __checkAuth(self, username: str):
        """Check user permissions

        :param username: user name

        :return: None if all OK else error message
        """
        if username == "anonymous":
            return "Cannot perform this operation as anonymous"
        if "SiteManager" not in self.getProperties():
            return "Not authorized"

    def setToken(self, requestParams: dict) -> dict:
        """Set token

        :param requestParams: request parameters
        """
        if error := self.__checkAuth(username := self.getUserName()):
            return {"success": "false", "error": error}

        res = PublisherClient().setToken(
            self.ELEMENT_TYPE,
            str(requestParams["name"][0]),
            str(requestParams["statusType"][0]),
            str(requestParams["status"][0]),
            str(requestParams["elementType"][0]),
            username,
            str(requestParams["lastCheckTime"][0]),
        )

        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        return {"success": "true", "result": res["Value"]}

    def setStatus(self, requestParams: dict) -> dict:
        """Set token

        :param requestParams: request parameters
        """
        if error := self.__checkAuth(username := self.getUserName()):
            return {"success": "false", "error": error}

        res = PublisherClient().setStatus(
            self.ELEMENT_TYPE,
            str(requestParams["name"][0]),
            str(requestParams["statusType"][0]),
            str(requestParams["status"][0]),
            str(requestParams["elementType"][0]),
            username,
            str(requestParams["lastCheckTime"][0]),
        )

        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        return {"success": "true", "result": res["Value"]}

    def _checkParameters(self, requestParams: dict, parametersToCheck: list):
        """Check incoming parameters

        :param requestParams: request parameters
        :param parametersToCheck: parameters to check

        :return: None if all is OK else error message
        """
        for fieldName in parametersToCheck:
            if not requestParams.get(fieldName):
                return f"Missing {fieldName}"

    def _getHistory(self, requestParams: dict) -> dict:
        """Get history

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "elementType", "statusType"]):
            return {"success": "false", "error": error}

        res = PublisherClient().getElementHistory(
            self.ELEMENT_TYPE,
            requestParams["name"],
            requestParams["elementType"],
            requestParams["statusType"],
        )

        if not res["OK"]:
            gLogger.error(res["Message"])
            return {"success": "false", "error": "error getting history"}

        history = [[r[0], str(r[1]), r[2]] for r in res["Value"]]

        return {"success": "true", "result": history, "total": len(history)}

    def _getPolicies(self, requestParams: dict) -> dict:
        """Get policies

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "statusType"]):
            return {"success": "false", "error": error}

        res = PublisherClient().getElementPolicies(
            self.ELEMENT_TYPE, requestParams["name"], requestParams["statusType"]
        )

        if not res["OK"]:
            gLogger.error(res["Message"])
            return {"success": "false", "error": "error getting policies"}

        policies = [[r[0], r[1], str(r[2]), str(r[3]), r[4]] for r in res["Value"]]

        return {"success": "true", "result": policies, "total": len(policies)}

    def _getDowntime(self, requestParams: dict) -> dict:
        """Get downtime

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "element", "elementType", "statusType"]):
            return {"success": "false", "error": error}

        res = PublisherClient().getDowntimes(
            str(requestParams["element"][-1]),
            str(requestParams["elementType"][-1]),
            str(requestParams["name"][-1]),
        )
        if not res["OK"]:
            gLogger.error(res["Message"])
            return {"success": "false", "error": "error getting downtimes"}

        downtimes = [[str(dt[0]), str(dt[1]), dt[2], dt[3], dt[4]] for dt in res["Value"]]

        return {"success": "true", "result": downtimes, "total": len(downtimes)}

    def _getTimeline(self, requestParams: dict) -> dict:
        """Get timeline

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "elementType", "statusType"]):
            return {"success": "false", "error": error}

        res = PublisherClient().getElementHistory(
            self.ELEMENT_TYPE,
            str(requestParams["name"][-1]),
            str(requestParams["elementType"][-1]),
            str(requestParams["statusType"][-1]),
        )

        if not res["OK"]:
            gLogger.error(res["Message"])
            return {"success": "false", "error": "error getting history"}

        history = []

        for status, dateEffective, reason in res["Value"]:

            # history.append( [ history[ -1 ][ 0 ], str( dateEffective - timedelta( seconds = 1 ) ), '' ] )

            history.append([status, str(dateEffective), reason])

        return {"success": "true", "result": history, "total": len(history)}

    def _getTree(self, requestParams: dict) -> dict:
        """Get timeline

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "elementType", "statusType"]):
            return {"success": "false", "error": error}

        pub = PublisherClient()
        res = PublisherClient().getTree(str(requestParams["elementType"][-1]), str(requestParams["name"][-1]))
        if not res["OK"]:
            gLogger.error(res["Message"])
            return {"success": "false", "error": "error getting tree"}
        res = res["Value"]

        siteName = list(res)[0]

        tree = [[siteName, None, None, None]]
        for k, v in res[siteName]["statusTypes"].items():
            tree.append([None, k, v, siteName])

        tree.append(["ces", None, None, siteName])
        for ce, ceDict in res[siteName]["ces"].items():
            tree.append([ce, None, None, "ces"])
            for k, v in ceDict.items():
                tree.append([None, k, v, ce])

        tree.append(["ses", None, None, siteName])
        for se, seDict in res[siteName]["ses"].items():
            tree.append([se, None, None, "ses"])
            for k, v in seDict.items():
                tree.append([None, k, v, se])

        return {"success": "true", "result": tree, "total": len(tree)}

    def _getInfo(self, requestParams: dict) -> dict:
        """Get timeline

        :param requestParams: request parameters
        """
        if error := self._checkParameters(requestParams, ["name", "element", "elementType", "statusType"]):
            return {"success": "false", "error": error}

        res = PublisherClient().getElementStatuses(
            str(requestParams["element"][-1]),
            str(requestParams["name"][-1]),
            str(requestParams["elementType"][-1]),
            str(requestParams["statusType"][-1]),
            None,
            None,
        )

        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        if not res["Value"]:
            return {"success": "false", "error": "Nothing found."}

        columns = res["Columns"]

        res = dict(zip(columns, res["Value"][0]))
        res["DateEffective"] = str(res["DateEffective"])
        res["LastCheckTime"] = str(res["LastCheckTime"])
        res["TokenExpiration"] = str(res["TokenExpiration"])

        return {"success": "true", "result": res, "total": len(res)}

    def _requestParams(self) -> dict:
        """
        We receive the request and we parse it, in this case, we are doing nothing,
        but it can be certainly more complex.
        """
        gLogger.always("!!!  PARAMS: ", repr(self.request.arguments))

        responseParams = {
            "element": None,
            "name": None,
            "elementType": None,
            "statusType": None,
            "status": None,
            "tokenOwner": None,
            "lastCheckTime": None,
            "action": None,
        }

        for key in responseParams:
            value = self.get_argument(key, "")
            if value:
                responseParams[key] = list(json.loads(value))

        return responseParams


class ResourceSummaryHandler(SummaryHandlerMix):

    ELEMENT_TYPE = "Resource"

    @asyncGen
    def web_getSelectionData(self):
        callback = yield self.threadTask(self._getSelectionData)
        self.finish(callback)

    @asyncGen
    def web_expand(self):
        callback = yield self.threadTask(self._expand)
        self.finish(callback)

    @asyncGen
    def web_action(self):
        callback = yield self.threadTask(self._action)
        self.finish(callback)

    @asyncGen
    def web_getResourceSummaryData(self):
        """This method returns the data required to fill the grid."""

        requestParams = self._requestParams()
        gLogger.info(requestParams)

        pub = PublisherClient()

        elementStatuses = yield self.threadTask(
            pub.getElementStatuses,
            self.ELEMENT_TYPE,
            requestParams["name"],
            requestParams["elementType"],
            requestParams["statusType"],
            requestParams["status"],
            requestParams["tokenOwner"],
        )
        if not elementStatuses["OK"]:
            self.finish({"success": "false", "error": elementStatuses["Message"]})
            return

        elementTree = collections.defaultdict(list)

        for element in elementStatuses["Value"]:

            elementDict = dict(zip(elementStatuses["Columns"], element))

            elementDict["DateEffective"] = str(elementDict["DateEffective"])
            elementDict["LastCheckTime"] = str(elementDict["LastCheckTime"])
            elementDict["TokenExpiration"] = str(elementDict["TokenExpiration"])

            elementTree[elementDict["Name"]].append(elementDict)

        elementList = []

        for elementValues in elementTree.values():

            if len(elementValues) == 1:
                elementList.append(elementValues[0])
            else:

                elementList.append(self.combine(elementValues))

        rssMachine = RSSMachine(None)

        yield self.threadTask(rssMachine.orderPolicyResults, elementList)

        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")

        self.finish({"success": "true", "result": elementList, "total": len(elementList), "date": timestamp})
